package pp.s1184725.boppi.test;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import org.junit.Test;

/**
 * Test cases for fixed-size arrays.
 * 
 * @author Frank Wibbelink
 */
public class ArrayTest {

	/**
	 * Correct array parsing
	 */
	@Test
	public void correctArrayParsing() {
		BoppiTests.parseFile("simpleArray.boppi");
		assertThat(BoppiTests.log, empty());
	}

	/**
	 * Arrays with syntax errors
	 */
	@Test
	public void wrongArrayParsing() {
		BoppiTests.parseString("var []int arr; 0");
		assertThat(BoppiTests.log, not(empty()));

		BoppiTests.parseString("var int a; a := 5; var [5][a]int arr; 0");
		assertThat(BoppiTests.log, not(empty()));

		BoppiTests.parseString("var [4]int arr; (arr)[5]");
		assertThat(BoppiTests.log, hasSize(1));
	}

	/**
	 * Correct array checking
	 */
	@Test
	public void correctArrayChecking() {
		BoppiTests.checkFile("simpleArray.boppi");
		assertThat(BoppiTests.log, is(empty()));
	}

	/**
	 * Arrays with type errors
	 */
	@Test
	public void wrongArrayChecking() {
		BoppiTests.checkString("var int a; a[5]");
		assertThat(BoppiTests.log, not(empty()));

		BoppiTests.checkString("var [5]int arr; arr := 5");
		assertThat(BoppiTests.log, not(empty()));

		BoppiTests.checkString("var [5]int arr; arr[4][2] := 5");
		assertThat(BoppiTests.log, not(empty()));

		BoppiTests.checkString("var int a; var int[5] arr; a := arr");
		assertThat(BoppiTests.log, not(empty()));
	}

	/**
	 * Correct array generation
	 */
	@Test
	public void correctArrayGeneration() {
		BoppiTests.compileAndRunFile("simpleArray.boppi");
		assertThat(BoppiTests.vm.getInterrupt(), is(0));
		assertThat(BoppiTests.log, is(empty()));
		assertThat(BoppiTests.out, is(arrayContaining("9", "H", "a", "Y")));
		
		BoppiTests.compileAndRunFile("complexArray.boppi");
		assertThat(BoppiTests.vm.getInterrupt(), is(0));
		assertThat(BoppiTests.log, is(empty()));
		assertThat(BoppiTests.out, is(arrayContaining("1337", "13", "2", "15")));
		
	}

	/**
	 * Out of bounds checking
	 */
	@Test
	public void wrongArrayGeneration() {
		BoppiTests.compileAndRunString("var [10]int arr; arr[10] := 5");
		assertThat(BoppiTests.vm.getInterrupt(), is(not(0)));
		
		BoppiTests.compileAndRunString("var [10]int arr; arr[-1]");
		assertThat(BoppiTests.vm.getInterrupt(), is(not(0)));
	}

}
